-- The addon namespace.
local addon, ns = ...

-- Define the manager class and put it in the addon namespace.
local Manager = {}
local ManagerMetatable = { __index = Manager }
ns.Manager = Manager

-- Constructor.
function Manager:new(shieldBarrierIcon, shieldBlockIcon)
	local self = {}
	setmetatable(self, ManagerMetatable)
	
	-- Initialize variables.
	self.Initialized = false
	self.Active = false
	self.PlayerName = UnitName("player") 
	self.CurrentRage = 0
	self.TotalBlocked = 0
	self.ShieldBlockActive = false
	self.ShieldBarrierActive = false
	self.BlockedAmounts = ns.LinkedList:new()
	self.ElapsedTime = 0.0
	self.ShieldBarrierIcon = shieldBarrierIcon
	self.ShieldBlockIcon = shieldBlockIcon

	-- Main frame (pun intended) for listening.
	self.Main =  CreateFrame("Frame", nil, UIParent)
	
	-- Event handler stuff. handlers are not fired when the addon is inactive, 
	-- listeners are. This is just my personal terminology.
	self.Handlers = {}
	self.Listeners = {}
	
	function self.Handlers.COMBAT_LOG_EVENT_UNFILTERED(sender, ...)
		if not self.ShieldBlockIcon.Active then
			return
		end
	
		local eventType = select(2, CombatLogGetCurrentEventInfo())
		local target = select(9, CombatLogGetCurrentEventInfo())
		local spellId = select(12, CombatLogGetCurrentEventInfo())
		
		if eventType == "SPELL_AURA_APPLIED" and spellId == self.ShieldBlockIcon.SpellId and target == self.PlayerName then
			self.ShieldBlockActive = true
			self.TotalBlocked = 0
		end 
		
		if eventType == "SPELL_AURA_REMOVED" and spellId == self.ShieldBlockIcon.SpellId and target == self.PlayerName then
			if not AuraUtil.FindAuraByName("Shield Block", "player") then
				self.ShieldBlockActive = false
			elseif AuraUtil.FindAuraByName("Shield Block", "player") then
				self.TotalBlocked = 0
			end			
		end
		
		if eventType == "SWING_DAMAGE" and target == self.PlayerName then
			local damage = select(12, CombatLogGetCurrentEventInfo()) or 0
			local blocked = select(16, CombatLogGetCurrentEventInfo()) or 0
			local absorbed = select(17, CombatLogGetCurrentEventInfo()) or 0

			self.BlockedAmounts:Add(damage + blocked + absorbed)
			
			if blocked and blocked > 0 and self.ShieldBlockActive then
				self.TotalBlocked = self.TotalBlocked + blocked
			end
		end
		
		if eventType == "SPELL_DAMAGE" and target == self.PlayerName then
			local damage = select(15, CombatLogGetCurrentEventInfo()) or 0
			local blocked = select(19, CombatLogGetCurrentEventInfo()) or 0
			local absorbed = select(20, CombatLogGetCurrentEventInfo()) or 0

			if (blocked and blocked > 0) or ShieldMaidSpellDatabase[spellId] then
				self.BlockedAmounts:Add(damage + blocked + absorbed)
			end
			
			if blocked and blocked > 0 then
				ShieldMaidSpellDatabase[spellId] = true
				
				if self.ShieldBlockActive then
					self.TotalBlocked = self.TotalBlocked + blocked
				end
			end
		end
		
		if eventType == "SWING_MISSED" and target == self.PlayerName then
			local missType = select(12, ...)
			local absorbed = select(15, ...) or 0
			if missType == "ABSORB" then
				self.BlockedAmounts:Add(absorbed)
			end
		end
		
		if eventType == "SPELL_MISSED" and target == self.PlayerName then
			local missType = select(15, ...)
			local absorbed = select(18, ...) or 0
			if missType == "ABSORB" and ShieldMaidSpellDatabase[spellId] then
				self.BlockedAmounts:Add(absorbed)
			end
		end
	end

	function self.Handlers.UNIT_POWER_UPDATE(sender, ...)
		local unitId = select(1, ...)
		if unitId == "player" then
			self.CurrentRage = UnitPower("player", SPELL_POWER_RAGE)
		end
	end

	function self.Handlers.PLAYER_REGEN_ENABLED(...)
		if ShieldMaidConfig.hiddenOutOfCombat then
			self.ShieldBarrierIcon:Hide()
			self.ShieldBlockIcon:Hide()
		end
	end

	function self.Handlers.PLAYER_REGEN_DISABLED(...)
		if ShieldMaidConfig.hiddenOutOfCombat then
			self.ShieldBarrierIcon:Show()
			self.ShieldBlockIcon:Show()
		end
	end	


	
	function self.Listeners.ADDON_LOADED(sender, ...)
		local name = select(1, ...)
		if name == addon then
			self:LoadDefaultConfig()
			self:LoadSpellDatabase()
			self:Options()
			self:CheckOptions()			
		end
	end
	
	function self.Listeners.PLAYER_LOGIN(...)
		self:Load()
		self:CheckOptions()
	end
	
	function self.Listeners.ACTIVE_TALENT_GROUP_CHANGED(...)
		self:Load()
	end
	
	return self
end

-- Returns an estimate of the maximum absorb value of Ignore Pain.
function Manager:CalculateMaxAbsorb()
	local maxhp = UnitHealthMax("player")
	return (maxhp * 0.3)
end

-- Calculates the estimated absorb value of Ignore Pain.
function Manager:CalculateEstimatedAbsorb()
	local strpain = GetSpellDescription(190456)
	strpain = string.gsub(strpain, ",", "")
	_, _, strpain = string.find(strpain, "(%d%d+)", 55)
	local estAbsorb = tonumber(strpain)
	if estAbsorb then
		return estAbsorb
	else
		return 0
	end	
end	

-- Calculates the estimated block based on entries from the past 6 seconds in the linked list.
-- At the same time, entries more than 6 seconds old are removed.
function Manager:CalculateEstimatedBlock()
	local blockDR = PaperDollFrame_GetArmorReduction(GetShieldBlock(), UnitEffectiveLevel("player")) / 100
	local critblockDR = blockDR * 2
	local sum = self.BlockedAmounts:Sum(6)
	local mastery = GetMasteryEffect()
	local criticalBlockChance = mastery / 100
	local criticalBlock,normalBlock
	
	if blockDR > 0.85 then blockDR = 0.85 end
	if critblockDR > 0.85 then critblockDR = 0.85 end
	criticalBlock = sum * criticalBlockChance * critblockDR
	normalBlock = sum * (1 - criticalBlockChance) * blockDR
	
	return criticalBlock + normalBlock
end

-- Tick function, called in updateInterval intervals.
function Manager:Tick()
	if self.ShieldBarrierIcon.Active and self.ShieldBlockIcon.Active then
		local estimatedAbsorb = self:CalculateEstimatedAbsorb()
		local estimatedBlock = self:CalculateEstimatedBlock()
		local estimatedMax = self:CalculateMaxAbsorb()
		self.ShieldBarrierIcon:Update(self.CurrentRage, estimatedAbsorb, estimatedBlock, estimatedMax)
		self.ShieldBlockIcon:Update(self.CurrentRage, self.TotalBlocked, estimatedAbsorb, estimatedBlock)
	end
end

-- Enables/disables the addon depending on class and spec. 
function Manager:Load()
	local class, _ = UnitClassBase("player")
	local spec = GetSpecialization()
	if class == "WARRIOR" and (spec == 3) then
	--if class == "WARRIOR" and (spec == 3 or not ShieldMaidConfig.tankOnly) then
		self.Active = true
		--C_Timer.After(3, function() self.Active = true end)
		
		if not self.ShieldBarrierIcon.Active and not self.ShieldBlockIcon.Active then
			self.ShieldBarrierIcon:Reload()
			self.ShieldBlockIcon:Reload()
			self.ShieldBarrierIcon:Show()
			self.ShieldBlockIcon:Show()
			self.ShieldBarrierIcon.Active = true
			self.ShieldBlockIcon.Active = true
	
			-- In case we reload or change spec in the middle of combat, we check for it here.
			if UnitAffectingCombat("player") or not ShieldMaidConfig.hiddenOutOfCombat then
				self.ShieldBarrierIcon:Show()
				self.ShieldBlockIcon:Show()
			else
				self.ShieldBarrierIcon:Hide()
				self.ShieldBlockIcon:Hide()
			end
			
			ns.Console.Write("|c0000FF00Loaded|r")
		end		
	elseif self.Active then
		self.Active = false
		self.ShieldBarrierIcon:Hide()
		self.ShieldBlockIcon:Hide()
		self.ShieldBlockIcon.Active = false
		self.ShieldBarrierIcon.Active = false
		ns.Console.Write("|c00FF0000Unloaded|r")
	end
end

-- Handler that makes sure the Tick function is called every ns.Config.updateInterval seconds.
function Manager:OnUpdateHandler(sender, seconds)
	if UnitAffectingCombat("player") then
		test = ShieldMaidConfig.updateIntervalCombat
	else
		test = ShieldMaidConfig.updateIntervalNoCombat
	end
	
	self.ElapsedTime = self.ElapsedTime + seconds
	if self.Active and self.ElapsedTime > test then
		self:Tick()
		self.ElapsedTime = self.ElapsedTime - test
	end
end

-- OnEvent handler that delegates to the correct handler. If inactive, we only handle ACTIVE_TALENT_GROUP_CHANGED.
function Manager:OnEventHandler(sender, event, ...)
	if self.Active and self.Handlers[event] then
		self.Handlers[event](sender, ...)
	end
	if self.Listeners[event] then
		self.Listeners[event](sender, ...)
	end
end

-- Registers update and event handlers.
function Manager:RegisterHandlers()
	for k, v in pairs(self.Handlers) do
		self.Main:RegisterEvent(k);
	end
	
	for k, v in pairs(self.Listeners) do
		self.Main:RegisterEvent(k);
	end

	self.Main:SetScript("OnUpdate", function(...) self:OnUpdateHandler(...) end)
	self.Main:SetScript("OnEvent", function(...) self:OnEventHandler(...) end)
end

-- Set the default configuration if no previous configuration has been saved.
function Manager:LoadSpellDatabase()
	if not ShieldMaidSpellDatabase then
		ShieldMaidSpellDatabase = {}
	end
end

-- Set the default configuration if no previous configuration has been saved.
function Manager:LoadDefaultConfig()
	if not ShieldMaidConfig then
		ShieldMaidConfig = {}
	end

	for k, v in pairs(ns.Config.Default) do
		if ShieldMaidConfig[k] == nil then
			ShieldMaidConfig[k] = v
		end
	end
end

-- Set the default configuration if no previous configuration has been saved.
function Manager:Reset()
	ShieldMaidConfig = {}
	for k, v in pairs(ns.Config.Default) do
		ShieldMaidConfig[k] = v
	end
	
	self.ShieldBarrierIcon:Reload()
	self.ShieldBlockIcon:Reload()
end

-- Locks the icons.
function Manager:SetLock(value)
	ShieldMaidConfig.locked = value
	if (value) then
		self.ShieldBarrierIcon:Lock()
		self.ShieldBlockIcon:Lock()
	else
		self.ShieldBarrierIcon:Unlock()
		self.ShieldBlockIcon:Unlock()
	end	
end

-- Sets the icon sizes.
function Manager:SetIconSize(size)
	ShieldMaidConfig.size = size
	self.ShieldBarrierIcon:Reload()
	self.ShieldBlockIcon:Reload()
end

-- Sets the icon scale.
function Manager:SetScale(scale)
	ShieldMaidConfig.scale = scale
	self.ShieldBarrierIcon:Reload()
	self.ShieldBlockIcon:Reload()
end

-- Sets the icon margin.
function Manager:SetMargin(margin)
	ShieldMaidConfig.margin = margin
	self.ShieldBarrierIcon:Reload()
	self.ShieldBlockIcon:Reload()
end

-- Sets a value indicating whether to hide the icons when out of combat.
function Manager:SetHiddenOutOfCombat(value)
	ShieldMaidConfig.hiddenOutOfCombat = value
	if UnitAffectingCombat("player") or not ShieldMaidConfig.hiddenOutOfCombat then
		self.ShieldBarrierIcon:Show()
		self.ShieldBlockIcon:Show()
	else
		self.ShieldBarrierIcon:Hide()
		self.ShieldBlockIcon:Hide()
	end
end

function Manager:SetUpdateIntervalCombat(value)
	if value < .01 then
		value = .01
	elseif value > .5 then
		value = .5
	end
	ShieldMaidConfig.updateIntervalCombat = value
end

function Manager:GetUpdateIntervalCombat()
	return ShieldMaidConfig.updateIntervalCombat
end

function Manager:SetSecondaryBarrier(value)
	ShieldMaidConfig.secondaryBarrier = value
end

function Manager:SetUpdateIntervalNoCombat(value)
	if value < .1 then
		value = .1
	elseif value > 2 then
		value = 2
	end
	ShieldMaidConfig.updateIntervalNoCombat = value
end

function Manager:GetUpdateIntervalNoCombat()
	return ShieldMaidConfig.updateIntervalNoCombat
end

function Manager:SetSecondaryBarrier(value)
	ShieldMaidConfig.secondaryBarrier = value
end

-- Sets a value indicating whether to show the icon frames.
function Manager:SetShowFrames(value)
	ShieldMaidConfig.showFrames = value
	self.ShieldBarrierIcon:Reload()
	self.ShieldBlockIcon:Reload()
end

-- Sets a value indicating whether to show the glow.
function Manager:SetShowGlow(value)
	ShieldMaidConfig.showGlow = value
	if not value then
		self.ShieldBarrierIcon:HideGlow()
		self.ShieldBlockIcon:HideGlow()
	end
end

function Manager:SetTruncatedNumbers(value)
	ShieldMaidConfig.truncatedNumbers = value
end

--[[function Manager:SetTankOnly(value)
	ShieldMaidConfig.tankOnly = value
	self:Load()
end]]--

-- Sets a value indicating whether to show the cooldown "clock" on icons.
function Manager:SetShowCooldown(value)
	ShieldMaidConfig.showCooldown = value
end

--Interface options panel
function Manager:Options()
	local smOptions = CreateFrame("Frame")
	smOptions.name = "Shield Maid"
	function smOptions.okay()
		xpcall(function () 
			local class, _ = UnitClassBase("player")
			local spec = GetSpecialization()
			if class == "WARRIOR" and spec == 3 then
				self.ShieldBarrierIcon:Reload()
				self.ShieldBlockIcon:Reload()
			end
			self:CheckOptions()
		end, geterrorhandler())
	end
	function smOptions.cancel()
		xpcall(function () 
			local class, _ = UnitClassBase("player")
			local spec = GetSpecialization()
			if class == "WARRIOR" and spec == 3 then
				self.ShieldBarrierIcon:Reload()
				self.ShieldBlockIcon:Reload()
			end
			self:CheckOptions()
		end, geterrorhandler())
	end
	InterfaceOptions_AddCategory(smOptions)

	local title = smOptions:CreateFontString(nil, "ARTWORK", "GameFontHighlightLarge")
	title:SetTextColor(1, 1, .20)
	title:SetPoint("CENTER", 0, 250)
	title:SetText(smOptions.name)


	ShieldMaidBarrierButton = CreateFrame("CheckButton", "ShieldMaidBarrierButton_GlobalName", smOptions, "ChatConfigCheckButtonTemplate")
	ShieldMaidBarrierButton:SetPoint("TOPLEFT", 32, -50)
	ShieldMaidBarrierButton_GlobalNameText:SetText("Display Secondary Ignore Pain Estimates")
	ShieldMaidBarrierButton.tooltip = "Check to display the estimated value of hitting Ignore Pain while Ignore Pain is already active."
	ShieldMaidBarrierButton:SetScript("OnClick", 
		function()
			if ShieldMaidBarrierButton:GetChecked() then
				self:SetSecondaryBarrier(true)
				PlaySound(856) -- Check Click Sound
			else
				self:SetSecondaryBarrier(false)
				PlaySound(857) -- Check Unclick Sound
			end
		end)
		
	ShieldMaidOOCButton = CreateFrame("CheckButton", "ShieldMaidOOCButton_GlobalName", smOptions, "ChatConfigCheckButtonTemplate")
	ShieldMaidOOCButton:SetPoint("TOPLEFT", 32, -80)
	ShieldMaidOOCButton_GlobalNameText:SetText("Hide while out of combat")
	ShieldMaidOOCButton.tooltip = "Check to display the icons only while in combat."
	ShieldMaidOOCButton:SetScript("OnClick", 
		function()
			if ShieldMaidOOCButton:GetChecked() then
				self:SetHiddenOutOfCombat(true)
				PlaySound(856) -- Check Click Sound
			else
				self:SetHiddenOutOfCombat(false)
				PlaySound(857) -- Check Unclick Sound
			end
		end)
	
	ShieldMaidLockButton = CreateFrame("CheckButton", "ShieldMaidLockButton_GlobalName", smOptions, "ChatConfigCheckButtonTemplate")
	ShieldMaidLockButton:SetPoint("TOPLEFT", 32, -110)
	ShieldMaidLockButton_GlobalNameText:SetText("Lock Frames")
	ShieldMaidLockButton.tooltip = "Check to lock the icons, preventing them from being moved."
	ShieldMaidLockButton:SetScript("OnClick", 
		function()
			if ShieldMaidLockButton:GetChecked() then
				self:SetLock(true)
				PlaySound(856) -- Check Click Sound
			else
				self:SetLock(false)
				PlaySound(857) -- Check Unclick Sound
			end
		end)
	
	ShieldMaidGlowButton = CreateFrame("CheckButton", "ShieldMaidGlowButton_GlobalName", smOptions, "ChatConfigCheckButtonTemplate")
	ShieldMaidGlowButton:SetPoint("TOPLEFT", 32, -140)
	ShieldMaidGlowButton_GlobalNameText:SetText("Show Icon Glow")
	ShieldMaidGlowButton.tooltip = "Check to display a glow around the icon which prevents the most damage per rage spent"
	ShieldMaidGlowButton:SetScript("OnClick", 
		function()
			if ShieldMaidGlowButton:GetChecked() then
				self:SetShowGlow(true)
				PlaySound(856) -- Check Click Sound
			else
				self:SetShowGlow(false)
				PlaySound(857) -- Check Unclick Sound
			end
		end)
		
	ShieldMaidFrameButton = CreateFrame("CheckButton", "ShieldMaidFrameButton_GlobalName", smOptions, "ChatConfigCheckButtonTemplate")
	ShieldMaidFrameButton:SetPoint("TOPLEFT", 32, -170)
	ShieldMaidFrameButton_GlobalNameText:SetText("Show Frame Border")
	ShieldMaidFrameButton.tooltip = "Check to display a border around each icon"
	ShieldMaidFrameButton:SetScript("OnClick", 
		function()
			if ShieldMaidFrameButton:GetChecked() then
				self:SetShowFrames(true)
				PlaySound(856) -- Check Click Sound
			else
				self:SetShowFrames(false)
				PlaySound(857) -- Check Unclick Sound
			end
		end)
		
	ShieldMaidTruncateButton = CreateFrame("CheckButton", "ShieldMaidTruncateButton_GlobalName", smOptions, "ChatConfigCheckButtonTemplate")
	ShieldMaidTruncateButton:SetPoint("TOPLEFT", 32, -200)
	ShieldMaidTruncateButton_GlobalNameText:SetText("Short Numbers")
	ShieldMaidTruncateButton.tooltip = "Check to abbreviate numbers down to 1 decimal place"
	ShieldMaidTruncateButton:SetScript("OnClick", 
		function()
			if ShieldMaidTruncateButton:GetChecked() then
				self:SetTruncatedNumbers(true)
				PlaySound(856) -- Check Click Sound
			else
				self:SetTruncatedNumbers(false)
				PlaySound(857) -- Check Unclick Sound
			end
		end)
		
	ShieldMaidCooldownButton = CreateFrame("CheckButton", "ShieldMaidCooldownButton_GlobalName", smOptions, "ChatConfigCheckButtonTemplate")
	ShieldMaidCooldownButton:SetPoint("TOPLEFT", 32, -230)
	ShieldMaidCooldownButton_GlobalNameText:SetText("Show Global Cooldown Clock")
	ShieldMaidCooldownButton.tooltip = "Check to show the GCD clock on Ignore Pains icon"
	ShieldMaidCooldownButton:SetScript("OnClick", 
		function()
			if ShieldMaidCooldownButton:GetChecked() then
				self:SetShowCooldown(true)
				PlaySound(856) -- Check Click Sound
			else
				self:SetShowCooldown(false)
				PlaySound(857) -- Check Unclick Sound
			end
		end)
	
	--[[
	ShieldMaidTankButton = CreateFrame("CheckButton", "ShieldMaidTankButton_GlobalName", smOptions, "ChatConfigCheckButtonTemplate")
	ShieldMaidTankButton:SetPoint("TOPLEFT", 32, -260)
	ShieldMaidTankButton_GlobalNameText:SetText("Enable only for Protection")
	ShieldMaidTankButton.tooltip = "Check to only display icons while set as protection. Uncheck to show icons for all specializations."
	ShieldMaidTankButton:SetScript("OnClick", 
		function()
			if ShieldMaidTankButton:GetChecked() then
				self:SetTankOnly(true)
				self:Load()	
				PlaySound(856) -- Check Click Sound
			else
				self:SetTankOnly(false)
				self:Load()	
				PlaySound(857) -- Check Unclick Sound
			end
		end)
	]]--
	
	local title1 = smOptions:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	title:SetTextColor(1, 1, .20)
	title1:SetPoint("TOPLEFT", 32, -310)
	title1:SetText("Changes to the below options may only be visible after hitting okay/cancel, or reloading UI")
	
	local titleSize = smOptions:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleSize:SetTextColor(1, 1, 1)
	titleSize:SetPoint("TOPLEFT", 35, -350)
	titleSize:SetText("Icon Size")	
	ShieldMaidSize = CreateFrame("EditBox", "Size", smOptions, "InputBoxTemplate")
	ShieldMaidSize:SetNumeric()
	ShieldMaidSize:ClearAllPoints()
	ShieldMaidSize:ClearFocus()
	ShieldMaidSize:SetSize(30, 30)
	ShieldMaidSize:SetPoint("TOPLEFT", 48, -360)
	ShieldMaidSize:SetText(tostring(ShieldMaidConfig.size))
	ShieldMaidSize:SetAutoFocus(false)
	ShieldMaidSize:SetCursorPosition(0)
	ShieldMaidSize:SetScript("OnEnterPressed", 
		function(self)
			local value = self:GetNumber()
			if (value > 10 and value < 100) then
				ShieldMaidConfig.size = value
			elseif value <= 10 then
				ShieldMaidConfig.size = 10
			elseif value >= 100 then
				ShieldMaidConfig.size = 100
			end
			self:ClearFocus()			
		end)
		
	local titleScale = smOptions:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleScale:SetTextColor(1, 1, 1)
	titleScale:SetPoint("TOPLEFT", 124, -350)
	titleScale:SetText("Icon Scale %")	
	ShieldMaidScale = CreateFrame("EditBox", "Size", smOptions, "InputBoxTemplate")
	ShieldMaidScale:SetNumeric()
	ShieldMaidScale:ClearAllPoints()
	ShieldMaidScale:ClearFocus()
	ShieldMaidScale:SetSize(30, 30)
	ShieldMaidScale:SetPoint("TOPLEFT", 148, -360)
	ShieldMaidScale:SetText(tostring(ShieldMaidConfig.scale * 100))
	ShieldMaidScale:SetAutoFocus(false)
	ShieldMaidScale:SetCursorPosition(0)
	ShieldMaidScale:SetScript("OnEnterPressed", 
		function(self)
			local value = self:GetNumber()
			if (value <= 10) then
				ShieldMaidConfig.scale = .1
			elseif (value > 10 and value < 200) then
				ShieldMaidConfig.scale = (value / 100)
			elseif (value >= 200) then
				ShieldMaidConfig.scale = 2
			end
			self:ClearFocus()
		end)
		
	local titleUpdate = smOptions:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleUpdate:SetTextColor(1, 1, 1)
	titleUpdate:SetPoint("TOPLEFT", 224, -350)
	titleUpdate:SetText("Updates/sec")	
	ShieldMaidUpdate = CreateFrame("EditBox", "Size", smOptions, "InputBoxTemplate")
	ShieldMaidUpdate:SetNumeric()
	ShieldMaidUpdate:ClearAllPoints()
	ShieldMaidUpdate:ClearFocus()
	ShieldMaidUpdate:SetSize(30, 30)
	ShieldMaidUpdate:SetPoint("TOPLEFT", 252, -360)
	ShieldMaidUpdate:SetText(tostring(1.00 / ShieldMaidConfig.updateIntervalCombat))
	ShieldMaidUpdate:SetAutoFocus(false)
	ShieldMaidUpdate:SetCursorPosition(0)
	ShieldMaidUpdate:SetScript("OnEnterPressed", 
		function(self)
			local value = self:GetNumber()
			if (value >= 2.00 and value <= 100.00) then
				ShieldMaidConfig.updateIntervalCombat = (1.00 / value)
			elseif value < 2.00 then
				ShieldMaidConfig.updateIntervalCombat = 0.5
			elseif value > 100.00 then
				ShieldMaidConfig.updateIntervalCombat = 0.05
			end
			self:ClearFocus()
		end)
	
	local titleUpdateOOC = smOptions:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleUpdateOOC:SetTextColor(1, 1, 1)
	titleUpdateOOC:SetPoint("TOPLEFT", 324, -350)
	titleUpdateOOC:SetText("Updates/sec OOC")	
	ShieldMaidUpdateOOC = CreateFrame("EditBox", "Size", smOptions, "InputBoxTemplate")
	ShieldMaidUpdateOOC:SetNumeric()
	ShieldMaidUpdateOOC:ClearAllPoints()
	ShieldMaidUpdateOOC:ClearFocus()
	ShieldMaidUpdateOOC:SetSize(30, 30)
	ShieldMaidUpdateOOC:SetPoint("TOPLEFT", 364, -360)
	ShieldMaidUpdateOOC:SetText(tostring(1.00 / ShieldMaidConfig.updateIntervalNoCombat))
	ShieldMaidUpdateOOC:SetAutoFocus(false)
	ShieldMaidUpdateOOC:SetCursorPosition(0)
	ShieldMaidUpdateOOC:SetScript("OnEnterPressed", 
		function(self)
			local value = self:GetNumber()
			if (value >= .5 and value <= 10.00) then
				ShieldMaidConfig.updateIntervalNoCombat = (1.00 / value)
			elseif value < 0.5 then
				ShieldMaidConfig.updateIntervalNoCombat = 2.00
			elseif value > 10.00 then
				ShieldMaidConfig.updateIntervalNoCombat = 0.1
			end
			self:ClearFocus()
		end)
	ShieldMaidBarrierButton:SetChecked(ShieldMaidConfig.secondaryBarrier)
	ShieldMaidOOCButton:SetChecked(ShieldMaidConfig.hiddenOutOfCombat)
	ShieldMaidLockButton:SetChecked(ShieldMaidConfig.locked)
	ShieldMaidGlowButton:SetChecked(ShieldMaidConfig.showGlow)	
	ShieldMaidFrameButton:SetChecked(ShieldMaidConfig.showFrames)
	ShieldMaidTruncateButton:SetChecked(ShieldMaidConfig.truncatedNumbers)
	ShieldMaidCooldownButton:SetChecked(ShieldMaidConfig.showCooldown)
	--ShieldMaidTankButton:SetChecked(ShieldMaidConfig.tankOnly)
	ShieldMaidSize:SetText(ShieldMaidConfig.size)
	ShieldMaidScale:SetText(ShieldMaidConfig.scale * 100)
	ShieldMaidUpdate:SetText(tostring(1.00 / ShieldMaidConfig.updateIntervalCombat))
	ShieldMaidUpdateOOC:SetText(tostring(1.00 / ShieldMaidConfig.updateIntervalNoCombat))	
end

function Manager:CheckOptions()
	ShieldMaidBarrierButton:SetChecked(ShieldMaidConfig.secondaryBarrier)
	ShieldMaidOOCButton:SetChecked(ShieldMaidConfig.hiddenOutOfCombat)
	ShieldMaidLockButton:SetChecked(ShieldMaidConfig.locked)
	ShieldMaidGlowButton:SetChecked(ShieldMaidConfig.showGlow)	
	ShieldMaidFrameButton:SetChecked(ShieldMaidConfig.showFrames)
	ShieldMaidTruncateButton:SetChecked(ShieldMaidConfig.truncatedNumbers)
	ShieldMaidCooldownButton:SetChecked(ShieldMaidConfig.showCooldown)	
	--ShieldMaidTankButton:SetChecked(ShieldMaidConfig.tankOnly)
	ShieldMaidSize:SetText(ShieldMaidConfig.size)
	ShieldMaidScale:SetText(ShieldMaidConfig.scale * 100)
	ShieldMaidUpdate:SetText(tostring(1.00 / ShieldMaidConfig.updateIntervalCombat))
	ShieldMaidUpdateOOC:SetText(tostring(1.00 / ShieldMaidConfig.updateIntervalNoCombat))	
end